<?php

namespace Drupal\search_overrides\EventSubscriber;

use Drupal\search_api_solr\Event\PreQueryEvent;
use Drupal\search_api_solr\Event\SearchApiSolrEvents;
use Drupal\search_api\Query\QueryInterface as SapiQueryInterface;
use Solarium\Core\Query\QueryInterface as SolariumQueryInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Alters the query where necessary to implement business logic.
 *
 * @package Drupal\search_overrides\EventSubscriber
 */
class SolrQueryAlterEventSubscriber implements EventSubscriberInterface {

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      SearchApiSolrEvents::PRE_QUERY => 'preQuery',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function preQuery(PreQueryEvent $event): void {
    $query = $event->getSearchApiQuery();
    $solarium_query = $event->getSolariumQuery();

    // Allow a method to review unmodified results.
    $ignore = \Drupal::request()->query->get('ignore_overrides');
    if (!empty($ignore)) {
      // Show unmodified results.
      return;
    }

    // Get search terms.
    $query_keys = $query->getKeys();
    // Exit if no query was performed.
    if (!$query_keys) {
      return;
    }

    $conjunction = $query->getParseMode()->getConjunction();
    if (is_array($query_keys)) {
      unset($query_keys['#conjunction']);
    }
    else {
      $query_keys = explode(' ', $query->getKeys());
    }

    // Should we only match the entire search string?
    $config = \Drupal::config('search_overrides.settings');
    $match_entire_string = $config->get('match_entire_string');

    // add a search for a phrase match if more than one term
    if (count($query_keys) > 1 && ($conjunction == 'AND')) {
      // Check if we should match only entire string or
      // entire string + each term individually
      if ($match_entire_string) {
        $query_keys = [implode(' ', $query_keys)];
      }
      else {
        // Add full query to the individual terms.
        array_unshift($query_keys, implode(' ', $query_keys));
      }
    }

    // Try to find an override for this query. We don't use these results to
    // retrieve the overrides because they wouldn't be ordered.
    $elQuery = \Drupal::entityQuery('search_override');
    $elevates = $elQuery->condition('query', $query_keys, 'IN')->accessCheck(TRUE)->execute();
    if (!$elevates) {
      // No record for this search query, so nothing to do.
      return;
    }

    // Initialize variables.
    $elSolrIds = [];
    $exSolrIds = [];
    $index = $query->getIndex();
    $current_lang = \Drupal::languageManager()->getCurrentLanguage()->getId();
    $elStorage = \Drupal::entityTypeManager()->getStorage('search_override');

    // Loop through search terms.
    foreach ($query_keys as $key) {
      $elQuery = \Drupal::entityQuery('search_override');
      $elevates = $elQuery->condition('query', $key)->accessCheck(TRUE)->execute();
      if (!$elevates) {
        // No record for this search term, so nothing to do.
        continue;
      }
      // Only load one entity per term, to reduce complexity
      // @todo prevent duplicates from being stored.
      $elevate = $elStorage->load(array_shift($elevates));

      // Handle items to elevate.
      $elevateIds = $elevate->getElevatedIds();
      foreach ($elevateIds as $id) {
        $elSolrIds[] = search_overrides_make_solr_id($index, $id, $current_lang);
      }

      // Handle exclusions.
      $excludeIds = $elevate->getExcludedIds();
      foreach ($excludeIds as $id) {
        $exSolrIds[] = search_overrides_make_solr_id($index, $id, $current_lang);
      }
    }

    $solarium_query->addParam('elevateIds', implode(',', $elSolrIds));
    $solarium_query->addParam('excludeIds', implode(',', $exSolrIds));
  }

}